@@ -1,4 +1,4 @@
-from mygrad.tensor_base import Tensor  # isort:skip  # avoid an import cycle
+from mygrad.tensor_base import Tensor, asarray, astensor  # isort:skip  # avoid an import cycle
 
 from mygrad.indexing_routines.funcs import *
 from mygrad.linalg.funcs import *
@@ -16,8 +16,8 @@ from mygrad.tensor_creation.funcs import *
 from mygrad.tensor_manip.array_shape.funcs import *
 from mygrad.tensor_manip.tiling.funcs import *
 from mygrad.tensor_manip.transpose_like.funcs import *
-from . import random
 
+from . import random
 from ._version import get_versions
 
 __version__ = get_versions()["version"]
