@@ -1,14 +1,17 @@
+from numbers import Real
+
 import numpy as np
 
-from mygrad import Tensor, log
+from mygrad.nnet.activations import softmax
 from mygrad.operation_base import Operation
+from mygrad.tensor_base import Tensor, asarray
 
 from ._utils import check_loss_inputs
 
 __all__ = ["softmax_focal_loss", "focal_loss"]
 
 
-class SoftmaxFocalLoss(Operation):
+class FocalLoss(Operation):
     r""" Returns the per-datum focal loss as described in https://arxiv.org/abs/1708.02002
     which is given by -ɑ(1-p)ˠlog(p).
 
@@ -29,11 +32,11 @@ class SoftmaxFocalLoss(Operation):
 
     scalar_only = True
 
-    def __call__(self, scores, targets, alpha, gamma):
+    def __call__(self, class_probs, targets, alpha, gamma):
         """
         Parameters
         ----------
-        scores : mygrad.Tensor, shape=(N, C)
+        class_probs : mygrad.Tensor, shape=(N, C)
             The C class scores for each of the N pieces of data.
 
         targets : Union[mygrad.Tensor, Sequence[int]], shape=(N,)
@@ -50,47 +53,64 @@ class SoftmaxFocalLoss(Operation):
         numpy.ndarray
             The per-datum focal loss.
         """
-        if isinstance(targets, Tensor):
+        if isinstance(targets, Tensor):  # pragma: nocover
             targets = targets.data
 
-        check_loss_inputs(scores, targets)
+        check_loss_inputs(class_probs, targets)
 
-        self.variables = (scores,)
-        scores = np.copy(scores.data if isinstance(scores, Tensor) else scores)
+        self.variables = (class_probs,)
+        self.label_locs = (range(len(class_probs)), targets)
 
-        max_scores = np.max(scores, axis=1, keepdims=True)
-        np.exp(scores - max_scores, out=scores)
-        scores /= np.sum(scores, axis=1, keepdims=True)
-        label_locs = (range(len(scores)), targets)
-
-        pc = scores[label_locs]
-        one_m_pc = 1 - pc + 1e-14  # correct domain for when gamma < 1 and pc == 1
+        class_probs = asarray(class_probs)
+        pc = class_probs[self.label_locs]
+        one_m_pc = np.clip(1 - pc, a_min=0, a_max=1)
         log_pc = np.log(pc)
 
-        loss = -(alpha * one_m_pc ** gamma * log_pc)
+        one_m_pc_gamma = one_m_pc ** gamma
+        loss = -(alpha * one_m_pc_gamma * log_pc)
 
-        self.back = scores
-        self.back[label_locs] -= 1
-        deriv = one_m_pc ** gamma - pc * gamma * one_m_pc ** (gamma - 1) * log_pc
-        self.back *= deriv[:, np.newaxis]
-        self.back *= alpha
+        self.back = np.zeros(class_probs.shape, dtype=np.float64)
+
+        if np.isclose(gamma, 0, atol=1e-15):
+            self.back[self.label_locs] -= alpha / pc
+            return loss
+
+        # dL/dp = -alpha * ( (1 - p)**g / p - g * (1 - p)**(g - 1) * log(p) )
+        #
+        # term 1: (1 - p)**g / p
+        term1 = one_m_pc_gamma / pc  # (1 - p)**g / p
+
+        # term 2: - g * (1 - p)**(g - 1) * log(p)
+        if np.isclose(gamma, 1, rtol=1e-15):
+            term2 = -log_pc
+        elif gamma < 1:
+            # For g < 1 and p -> 1, the 2nd term -> 0 via L'Hôpital's rule
+            term2 = np.zeros(pc.shape, dtype=np.float64)
+            pc_not_1 = ~np.isclose(one_m_pc, 0, atol=1e-15)
+            term2[pc_not_1] = (
+                -gamma * one_m_pc[pc_not_1] ** (gamma - 1) * log_pc[pc_not_1]
+            )
+        else:
+            term2 = -gamma * one_m_pc ** (gamma - 1) * log_pc
+
+        self.back[self.label_locs] -= alpha * (term1 + term2)
         return loss
 
     def backward_var(self, grad, index, **kwargs):
-        return grad[:, np.newaxis] * self.back
+        self.back[self.label_locs] *= grad
+        return self.back
 
 
-def softmax_focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
-    r"""
-    Applies the softmax normalization to the input scores before computing the
-    per-datum focal loss.
+def focal_loss(class_probs, targets, *, alpha=1, gamma=0, constant=False):
+    r""" Return the per-datum focal loss.
 
     Parameters
     ----------
-    scores : mygrad.Tensor, shape=(N, C)
-        The C class scores for each of the N pieces of data.
+    class_probs : mygrad.Tensor, shape=(N, C)
+        The C class probabilities for each of the N pieces of data.
+        Each value is expected to lie on (0, 1]
 
-    targets : array_like, shape=(N,)
+    targets : Sequence[int], shape=(N,)
         The correct class indices, in [0, C), for each datum.
 
     alpha : Real, optional (default=1)
@@ -98,6 +118,7 @@ def softmax_focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
 
     gamma : Real, optional (default=0)
         The ɣ focusing parameter. Note that for Ɣ=0 and ɑ=1, this is cross-entropy loss.
+        Must be a non-negative value.
 
     constant : bool, optional(default=False)
         If ``True``, the returned tensor is a constant (it
@@ -113,6 +134,7 @@ def softmax_focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
     The formulation for the focal loss introduced in https://arxiv.org/abs/1708.02002.
     It is given by -ɑ(1-p)ˠlog(p).
 
+
     The focal loss for datum-:math:`i` is given by
 
     .. math::
@@ -124,13 +146,18 @@ def softmax_focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
 
     It is recommended in the paper that you normalize by the number of foreground samples.
     """
+    if not isinstance(gamma, Real) or gamma < 0:
+        raise ValueError(f"`gamma` must be a non-negative number, got: {gamma}")
+
     return Tensor._op(
-        SoftmaxFocalLoss, scores, op_args=(targets, alpha, gamma), constant=constant
+        FocalLoss, class_probs, op_args=(targets, alpha, gamma), constant=constant
     )
 
 
-def focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
-    r""" Return the per-datum focal loss.
+def softmax_focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
+    r"""
+    Applies the softmax normalization to the input scores before computing the
+    per-datum focal loss.
 
     Parameters
     ----------
@@ -145,6 +172,7 @@ def focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
 
     gamma : Real, optional (default=0)
         The ɣ focusing parameter. Note that for Ɣ=0 and ɑ=1, this is cross-entropy loss.
+        Must be a non-negative value.
 
     constant : bool, optional(default=False)
         If ``True``, the returned tensor is a constant (it
@@ -160,7 +188,6 @@ def focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
     The formulation for the focal loss introduced in https://arxiv.org/abs/1708.02002.
     It is given by -ɑ(1-p)ˠlog(p).
 
-
     The focal loss for datum-:math:`i` is given by
 
     .. math::
@@ -172,11 +199,6 @@ def focal_loss(scores, targets, *, alpha=1, gamma=0, constant=False):
 
     It is recommended in the paper that you normalize by the number of foreground samples.
     """
-    if isinstance(targets, Tensor):
-        targets = targets.data
-
-    check_loss_inputs(scores, targets)
-
-    label_locs = (range(len(targets)), targets)
-    pc = scores[label_locs]
-    return -(alpha * (1 - pc + 1e-14) ** gamma * log(pc, constant=constant))
+    return focal_loss(
+        softmax(scores), targets=targets, alpha=alpha, gamma=gamma, constant=constant
+    )
