@@ -5,6 +5,7 @@ etc., are bound to the Tensor class in ``mygrad.__init__.py``.
 """
 
 from functools import wraps
+from numbers import Number
 from typing import Optional, Set, Type, Union
 
 import numpy as np
@@ -19,6 +20,7 @@ from mygrad.math.arithmetic.ops import (
     Negative,
     Positive,
     Power,
+    Square,
     Subtract,
 )
 from mygrad.operation_base import BroadcastableOp, Operation
@@ -26,7 +28,168 @@ from mygrad.tensor_core_ops.indexing import GetItem, SetItem
 from mygrad.tensor_manip.array_shape.ops import Flatten, Reshape
 from mygrad.tensor_manip.transpose_like.ops import Tensor_Transpose_Property
 
-__all__ = ["Tensor"]
+__all__ = ["Tensor", "asarray"]
+
+
+def asarray(a, dtype=None, order=None) -> np.ndarray:
+    """Convert the input to an array.
+
+    This docstring is adapted from that of ``numpy.asarray``
+
+    Parameters
+    ----------
+    a : array_like
+        Input data, in any form - including a mygrad tensor - that can be converted to an array. This
+        includes lists, lists of tuples, tuples, tuples of tuples, tuples
+        of lists and ndarrays.
+
+    dtype : data-type, optional
+        By default, the data-type is inferred from the input data.
+
+    order : {'C', 'F'}, optional
+        Whether to use row-major (C-style) or
+        column-major (Fortran-style) memory representation.
+        Defaults to 'C'.
+
+    Returns
+    -------
+    out : ndarray
+        Array interpretation of `a`.  No copy is performed if the input
+        is already an ndarray with matching dtype and order.  If `a` is a
+        subclass of ndarray, a base class ndarray is returned.
+
+    Examples
+    --------
+    Convert a list into an array:
+
+    >>> import mygrad as mg
+    >>> a = [1, 2]
+    >>> mg.asarray(a)
+    array([1, 2])
+
+    Convert a tensor into an array. No copy of the
+    underlying numpy array is created:
+
+    >>> t = mg.Tensor([1, 2.])
+    >>> mg.asarray(t)
+    array([1., 2.])
+    >>> t.data is np.asarray(t))
+    True
+
+    Existing arrays are not copied:
+
+    >>> a = np.array([1, 2])
+    >>> mg.asarray(a) is a
+    True
+
+    If `dtype` is set, array is copied only if dtype does not match:
+
+    >>> a = np.array([1, 2], dtype=np.float32)
+    >>> mg.asarray(a, dtype=np.float32) is a
+    True
+    >>> mg.asarray(a, dtype=np.float64) is a
+    False
+
+    Contrary to `asanyarray`, ndarray subclasses are not passed through:
+
+    >>> issubclass(np.recarray, np.ndarray)
+    True
+    >>> a = np.array([(1.0, 2), (3.0, 4)], dtype='f4,i4').view(np.recarray)
+    >>> mg.asarray(a) is a
+    False
+    >>> np.asanyarray(a) is a
+    True
+    """
+    return np.asarray(a, dtype=dtype, order=order)
+
+
+def astensor(t, dtype=None, constant=None) -> "Tensor":
+    """Convert the input to a tensor.
+
+    A tensor `t` is returned unchanged - its gradient and computational
+    graph state preserved - if dtype and constant are compatible.
+    A copy of the underlying numpy array is created only if dtype is
+    incompatible.
+
+    Parameters
+    ----------
+    t : array_like
+        Input data, in any form that can be converted to a tensor. This
+        includes lists, lists of tuples, tuples, tuples of tuples, tuples
+        of lists and ndarrays.
+
+    dtype : data-type, optional
+        By default, the data-type is inferred from the input data.
+
+    constant : Optional[bool]
+        By default, `constant` is inferred from `t` if `t` is a tensor,
+        otherwise it defaults to `False`.
+
+    Returns
+    -------
+    out : Tensor
+        Tensor interpretation of `a`.  No copy is performed if the input
+        is already a tensor with matching dtype and constant-flag.
+
+    Examples
+    --------
+    Convert a list into an array:
+
+    >>> import mygrad as mg
+    >>> import numpy as np
+    >>> t = [1, 2]
+    >>> mg.astensor(t)
+    Tensor([1, 2])
+
+    Convert an array into a tensor. No copy of the
+    underlying numpy array is created:
+
+    >>> a = np.array([1, 2.])
+    >>> mg.astensor(a)
+    Tensor([1., 2.])
+    >>> a is mg.astensor(a).data
+    True
+
+    Existing tensors are not copied and their gradients and
+    computational graphs are preserved:
+
+    >>> t = 2 * mg.Tensor([1, 2])
+    >>> t2 = mg.astensor(t)
+    >>> t is t2
+    True
+    >>> t.creator is t2.creator
+    True
+
+    If `dtype` is set, a new tensor is created - with copied data - only
+    if dtype does not match:
+
+    >>> t = mg.Tensor([1, 2], dtype=np.float32)
+    >>> mg.astensor(t, dtype=np.float32) is t
+    True
+    >>> mg.astensor(t, dtype=np.float64) is t
+    False
+
+    If `constant` is set, a new tensor is created (with no copy
+    of the underlying data) only if constant doesn't match.
+
+    >>> t = mg.Tensor([1, 2], constant=False)
+    >>> mg.astensor(t, constant=False) is t
+    True
+    >>> mg.astensor(t, constant=True) is t
+    False
+    >>> mg.astensor(t, constant=True).data is t.data
+    True
+    """
+    if (
+        isinstance(t, Tensor)
+        and (constant is None or t.constant is constant)
+        and (dtype is None or (t.dtype == np.dtype(dtype)))
+    ):
+        return t
+    else:
+        if constant is None:
+            constant = t.constant if isinstance(t, Tensor) else False
+        return Tensor(t, dtype=dtype, constant=constant)
 
 
 class Tensor:
@@ -139,15 +302,14 @@ class Tensor:
             The operation-instance whose forward pass produced `self`. Should not
             be set manually by users.
         """
-        assert isinstance(constant, bool)
+        if not isinstance(constant, bool):
+            raise TypeError(f"`constant` must be a boolean value, got: {constant}")
+
         self._scalar_only = _scalar_only
         self._creator = _creator  # type: Union[None, Operation]
 
-        if isinstance(x, Tensor):
-            self.data = x.data
-        else:
-            self.data = np.asarray(x, dtype=dtype)
-            self._check_valid_dtype(self.data.dtype)
+        self.data = np.asarray(x, dtype=dtype)
+        self._check_valid_dtype(self.data.dtype)
 
         self.grad = None  # type: Union[None, np.ndarray]
         self._constant = constant
@@ -327,7 +489,7 @@ class Tensor:
             return
 
         if grad is not None:
-            self.grad = np.asarray(grad.data if isinstance(grad, Tensor) else grad)
+            self.grad = asarray(grad)
             if is_invalid_gradient(self.grad):
                 raise InvalidGradient(
                     f"An invalid gradient-value was passed to "
@@ -443,7 +605,7 @@ class Tensor:
         if not clear_graph:
             assert isinstance(seen, set)
 
-            if self in seen:
+            if id(self) in seen:
                 return
 
         creator = self._creator
@@ -452,8 +614,9 @@ class Tensor:
             # marks tensor as "visited" during clear-graph traversal
             self._creator = None
         else:
-            # marks tensor as "visited" during null-gradients graph traversal
-            seen.add(self)
+            # marks tensor as "visited" during null-gradients graph traversal. Tensors are unhashable,
+            # so we use their ids, since we actually want to be relying on their uniqueness.
+            seen.add(id(self))
 
         for var in creator.variables:  # type: Tensor
             if clear_graph:
@@ -508,7 +671,7 @@ class Tensor:
         return self._scalar_only
 
     @property
-    def constant(self):
+    def constant(self) -> bool:
         """ If ``True``, this tensor is a constant; it will not propagate any gradient.
 
         Additionally, any tensor that is a descendant of constant tensors will also
@@ -574,11 +737,14 @@ class Tensor:
         return self.data.__contains__(item)
 
     def __getitem__(self, item):
+        return self._op(GetItem, self, op_args=(item,))
+
+    def __iter__(self):
         # In the same way that numpy doesn't let you iterate over 0-dimensional
         # arrays, don't allow iteration over 0-dimensional arrays.
         if self.ndim == 0:
             raise TypeError("iteration over a 0-d tensor")
-        return self._op(GetItem, self, op_args=(item,))
+        return iter(self[n] for n in range(len(self)))
 
     def _mirror_tensor(self, tensor: "Tensor"):
         """ *Dev use only*
@@ -672,6 +838,14 @@ class Tensor:
         return self._op(MatMul, other, self)
 
     def __pow__(self, other):
+        if isinstance(other, Number) or (
+            isinstance(other, np.ndarray) and other.ndim == 0
+        ):
+            if other == 1:
+                return self._op(Positive, self)
+            elif other == 2:
+                return self._op(Square, self)
+
         return self._op(Power, self, other)
 
     def __rpow__(self, other):
@@ -695,20 +869,20 @@ class Tensor:
         -------
         Tensor
         """
-        copy = Tensor(
-            np.copy(self.data),
-            _creator=None,
-            constant=self.constant,
-            _scalar_only=self._scalar_only,
-        )
-        copy.grad = np.copy(self.grad) if self.grad is not None else None
-        return copy
+        return self.copy()
 
-    def copy(self):
+    def copy(self, constant: Optional[bool] = None) -> "Tensor":
         """ Produces a copy of ``self`` with ``copy.creator=None``.
 
         Copies of the underlying numpy data array and gradient array are created.
 
+        No information regarding the tensor's participation in the computational
+        graph are copied.
+
+        Parameters
+        ----------
+        constant : Optional[bool]
+
         Returns
         -------
         Tensor
@@ -727,7 +901,14 @@ class Tensor:
         >>> y_copy.creator is None
         True
         """
-        return self.__copy__()
+        copy = Tensor(
+            np.copy(self.data),
+            _creator=None,
+            constant=(self.constant if constant is None else constant),
+            _scalar_only=self._scalar_only,
+        )
+        copy.grad = np.copy(self.grad) if self.grad is not None else None
+        return copy
 
     def item(self):
         """ Copy an element of a tensor to a standard Python scalar and return it.
@@ -958,6 +1139,25 @@ class Tensor:
         """
         return self._op(Tensor_Transpose_Property, self)
 
+    def __eq__(self, other):
+        return np.ndarray.__eq__(
+            self.data, other.data if isinstance(other, Tensor) else other
+        )
+
+    def __ne__(self, other):
+        return np.ndarray.__ne__(
+            self.data, other.data if isinstance(other, Tensor) else other
+        )
+
+    def __imatmul__(self, other):
+        raise TypeError(
+            "In-place matrix multiplication is not (yet) supported. "
+            "Use 'a = a @ b' instead of 'a @= b'"
+        )
+
+    def __array__(self, dtype=None):
+        return np.asarray(self.data, dtype)
+
 
 # set all comparison operators - mirrors ndarray methods
 def tensor_to_array_wrapper(func):
@@ -968,5 +1168,5 @@ def tensor_to_array_wrapper(func):
     return wrapped
 
 
-for op in ("__lt__", "__le__", "__gt__", "__ge__", "__eq__", "__ne__"):
+for op in ("__lt__", "__le__", "__gt__", "__ge__"):
     setattr(Tensor, op, tensor_to_array_wrapper(getattr(np.ndarray, op)))
